@@ -17,7 +17,7 @@
 ** Linux-port:  June 2000
 ** Modified: 	May 2001 - Ported to kernel 2.4
 ** --------------------------------------------------------------------------
-** Copyright (C) 2000-2018 Gerlof Langeveld
+** Copyright (C) 2000-2024 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -38,43 +38,43 @@
 ** For every cycle (so after another interval) the ENGINE calls various 
 ** functions as shown below:
 **
-** +---------------------------------------------------------------------+
-** |                           E  N  G  I  N  E                          |
-** |                                                                     |
-** |                                                                     |
-** |    _____________________await interval-timer_____________________   |
-** |   |                                                              ^  |
-** |   |      ________       ________      ________      ________     |  |
-** |   |     ^        |     ^        |    ^        |    ^        |    |  |
-** +---|-----|--------|-----|--------|----|--------|----|--------|----|--+
-**     |     |        |     |        |    |        |    |        |    |
-**  +--V-----|--+  +--V-----|--+  +--V----|--+  +--V----|--+  +--V----|-+  
-**  |           |  |           |  |          |  |          |  |         |
-**  | photosyst |  | photoproc |  |   acct   |  | deviate  |  |  print  |
-**  |           |  |           |  |photoproc |  |  ...syst |  |         |
-**  |           |  |           |  |          |  |  ...proc |  |         |
-**  +-----------+  +-----------+  +----------+  +----------+  +---------+  
-**        ^              ^             ^              ^            |
-**        |              |             |              |            |
-**        |              |             |              V            V 
-**      ______       _________     __________     ________     _________
-**     /      \     /         \   /          \   /        \   /         \
-**      /proc          /proc       accounting       task       screen or
-**                                    file        database        file
-**     \______/     \_________/   \__________/   \________/   \_________/
+** +------------------------------------------------------------------------+
+** |                           E  N  G  I  N  E                             |
+** |                                                                        |
+** |                                                                        |
+** |    _____________________await interval-timer________________________   |
+** |   |                                                                 ^  |
+** |   |    _______    _______    _______       ________      ________   |  |
+** |   |   ^       |  ^       |  ^       |     ^        |    ^        |  |  |
+** +---|---|-------|--|-------|--|-------|-----|--------|----|--------|--|--+
+**     |   |       |  |       |  |       |     |        |    |        |  |
+**  +--V-----+  +--V----+  +--V----+  +--V--------+  +--V-------+  +--V-----+  
+**  |        |  |       |  |       |  |           |  | deviate  |  |        |
+**  | photo  |  | photo |  | photo |  |   acct    |  | ..cgroup |  | print  |
+**  | cgroup |  | syst  |  | proc  |  | photoproc |  | ..syst   |  |        |
+**  |        |  |       |  |       |  |           |  | ..proc   |  |        |
+**  +--------+  +-------+  +-------+  +-----------+  +----------+  +--------+
+**      ^           ^          ^            ^              |            |
+**      |           |          |            |              |            |
+**      |           |          |            V              V            V
+**    _______     _____      _____      __________     ________     _________
+**   /       \   /     \    /     \    /          \   /        \   /         \
+**    /sys/fs     /proc      /proc      accounting       task       screen or
+**    /cgroup                              file        database        file
+**   \_______/   \_____/    \_____/    \__________/   \________/   \_________/
+**
+**    -	photocgroup()
+**	Takes a snapshot of the counters related to resource usage on
+** 	cgroup-level v2 (cpu, disk, memory).
 **
 **    -	photosyst()
 **	Takes a snapshot of the counters related to resource-usage on
 ** 	system-level (cpu, disk, memory, network).
-**	This code is UNIX-flavor dependent; in case of Linux the counters
-**	are retrieved from /proc.
 **
 **    -	photoproc()
 **	Takes a snapshot of the counters related to resource-usage of
 **	tasks which are currently active. For this purpose the whole
 **	task-list is read.
-**	This code is UNIX-flavor dependent; in case of Linux the counters
-**	are retrieved from /proc.
 **
 **    -	acctphotoproc()
 **	Takes a snapshot of the counters related to resource-usage of
@@ -82,11 +82,15 @@
 **	For this purpose all new records in the accounting-file are read.
 **
 ** When all counters have been gathered, functions are called to calculate
-** the difference between the current counter-values and the counter-values
-** of the previous cycle. These functions operate on the system-level
-** as well as on the task-level counters. 
-** These differences are stored in a new structure(-table). 
+** the difference between the current counter values and the counter values
+** of the previous cycle. These functions operate on cgroup level, system level
+** as well as on task level.
+** These differences are stored in a new structure (table). 
 **
+**    -	deviatcgroup()
+**	Calculates the differences between the current cgroup-level
+** 	counters and the corresponding counters of the previous cycle.
+
 **    -	deviatsyst()
 **	Calculates the differences between the current system-level
 ** 	counters and the corresponding counters of the previous cycle.
@@ -132,12 +136,14 @@
 #include <sys/resource.h>
 #include <regex.h>
 #include <glib.h>
+#include <sys/inotify.h>
 
 #include "atop.h"
 #include "acctproc.h"
 #include "ifprop.h"
 #include "photoproc.h"
 #include "photosyst.h"
+#include "cgroups.h"
 #include "showgeneric.h"
 #include "showlinux.h"
 #include "parseable.h"
@@ -145,8 +151,8 @@
 #include "gpucom.h"
 #include "netatop.h"
 
-#define	allflags  "ab:cde:fghijklmnopqrstuvwxyz:1ABCDEFGHIJ:KL:MNOP:QRSTUVWXYZ"
-#define	MAXFL		64      /* maximum number of command-line flags  */
+#define	allflags  "ab:cde:fghijklmnopqrstuvwxyz:123456789ABCDEFGHIJ:KL:MNOP:QRSTUVWXYZ"
+#define	MAXFL		84      /* maximum number of command-line flags  */
 
 /*
 ** declaration of global variables
@@ -158,6 +164,9 @@ time_t 		curtime;	/* timing info				*/
 unsigned long	interval = 10;
 unsigned long 	sampcnt;
 char		screen;
+int		fdinotify = -1;	/* inotify fd for twin mode  		*/
+pid_t		twinpid;	/* PID of lower half for twin mode	*/
+char		twindir[RAWNAMESZ] = "/tmp";
 int		linelen  = 80;
 char		acctreason;	/* accounting not active (return val) 	*/
 char		rawname[RAWNAMESZ];
@@ -205,6 +214,8 @@ static unsigned int 	nsamples = 0xffffffff;
 static char		midnightflag;
 static char		rawwriteflag;
 
+char			twinmodeflag;
+
 /*
 ** interpretation of defaults-file /etc/atoprc and $HOME/.atop
 */
@@ -219,6 +230,7 @@ static struct {
 	int	sysonly;
 } manrc[] = {
 	{	"flags",		do_flags,		0, },
+	{	"twindir",		do_twindir,		0, },
 	{	"interval",		do_interval,		0, },
 	{	"linelen",		do_linelength,		0, },
 	{	"username",		do_username,		0, },
@@ -272,6 +284,8 @@ static struct {
 ** internal prototypes
 */
 static void	engine(void);
+static void	twinprepare(void);
+static void	twinclean(void);
 
 int
 main(int argc, char *argv[])
@@ -357,14 +371,6 @@ main(int argc, char *argv[])
 				vis.show_samp = rawwrite;
 				break;
 
-			   case 'B':		/* bar graphs ?               */
-				displaymode = 'D';
-				break;
-
-			   case 'H':		/* bar graphs ?               */
-				barmono = 1;
-				break;
-
 			   case 'r':		/* reading of raw data ?      */
 				if (optind < argc)
 				{
@@ -372,8 +378,7 @@ main(int argc, char *argv[])
 					{
 						if (strlen(argv[optind]) == 1)
 						{
-							strcpy(rawname,
-								"/dev/stdin");
+							strcpy(rawname, "/dev/stdin");
 							optind++;
 						}
 					}
@@ -388,6 +393,29 @@ main(int argc, char *argv[])
 				rawreadflag++;
 				break;
 
+			   case 't':		/* twin mode ?		      */
+				// optional absolute path name of directory?
+				if (optind < argc)
+				{
+					if (*(argv[optind]) == '/')
+					{
+						strncpy(twindir, argv[optind],
+								RAWNAMESZ-1);
+						optind++;
+					}
+				}
+
+				twinmodeflag++;
+				break;
+
+			   case 'B':		/* bar graphs ?               */
+				displaymode = 'D';
+				break;
+
+			   case 'H':		/* bar graphs ?               */
+				barmono = 1;
+				break;
+
 			   case 'S':		/* midnight limit ?           */
 				midnightflag++;
 				break;
@@ -423,7 +451,7 @@ main(int argc, char *argv[])
 				linelen = atoi(optarg);
 				break;
 
-                           case MALLPROC:	/* all processes per sample ? */
+                           case MALLACTIVE:	/* all processes/cgroups ? */
 				deviatonly = 0;
 				break;
 
@@ -513,6 +541,17 @@ main(int argc, char *argv[])
 	pagesize	= sysconf(_SC_PAGESIZE);
 	pidwidth	= getpidwidth();
 
+	/*
+	** check if twin mode wanted with two atop processes:
+	**
+	** - lower half: gather statistics and write to raw file
+	** - upper half: read statistics and present to user
+	**
+	** consistency checks
+	*/
+	if (twinmodeflag)
+		twinprepare();
+
 	/*
 	** check if raw data from a file must be viewed
 	*/
@@ -522,11 +561,6 @@ main(int argc, char *argv[])
 		cleanstop(0);
 	}
 
-	/*
-	** determine start-time for gathering current statistics
-	*/
-	curtime = getboot() / hertz;
-
 	/*
 	** be sure to be leader of an own process group when
 	** running as a daemon (or at least: when not interactive);
@@ -535,6 +569,11 @@ main(int argc, char *argv[])
 	if (rawwriteflag)
 		(void) setpgid(0, 0);
 
+	/*
+	** determine start-time for gathering current statistics
+	*/
+	curtime = getboot() / hertz;
+
 	/*
 	** catch signals for proper close-down
 	*/
@@ -592,6 +631,11 @@ main(int argc, char *argv[])
         if (! droprootprivs() )
 		mcleanstop(42, "failed to drop root privs\n");
 
+	/*
+	** determine if cgroups v2 is supported
+	*/
+	cgroupv2support();
+
 	/*
 	** start the engine now .....
 	*/
@@ -611,6 +655,13 @@ engine(void)
 	struct sigaction 	sigact;
 	static time_t		timelimit;
 
+	/*
+	** reserve space for cgroup-level statistics
+	*/
+	static struct cgchainer	*devcstat;
+	int			ncgroups = 0;
+	int			npids    = 0;
+
 	/*
 	** reserve space for system-level statistics
 	*/
@@ -738,7 +789,7 @@ engine(void)
 			gpupending = gpud_statrequest();
 
 		/*
-		** take a snapshot of the current system-level statistics 
+		** take a snapshot of the current system-level metrics 
 		** and calculate the deviations (i.e. calculate the activity
 		** during the last sample)
 		*/
@@ -746,7 +797,14 @@ engine(void)
 		cursstat = presstat;
 		presstat = hlpsstat;
 
-		photosyst(cursstat);	/* obtain new counters      */
+		photosyst(cursstat);	/* obtain new system-level counters */
+
+		/*
+		** take a snapshot of the current cgroup-level metrics 
+		** when cgroups v2 supported
+		*/
+		if ( (supportflags&CGROUPV2) )
+			photocgroup();
 
 		/*
 		** receive and parse response from atopgpud
@@ -790,6 +848,7 @@ engine(void)
 		deviatsyst(cursstat, presstat, devsstat,
 				curtime-pretime > 0 ? curtime-pretime : 1);
 
+
 		/*
 		** take a snapshot of the current task-level statistics 
 		** and calculate the deviations (i.e. calculate the activity
@@ -876,23 +935,34 @@ engine(void)
 		 	             gp,       nrgpuproc);
 
 		/*
-		** calculate deviations
+		** calculate process-level deviations
 		*/
-		deviattask(curtpres,  ntaskpres, curpexit,  nprocexit,
-		                      &devtstat, devsstat);
+		deviattask(curtpres, ntaskpres, curpexit, nprocexit,
+		                     &devtstat, devsstat);
 
-		if (supportflags & NETATOPBPF) {
+		if (supportflags & NETATOPBPF)
+		{
 			g_hash_table_destroy(ghash_net);
 			ghash_net = NULL;
 		}
 
 		/*
-		** activate the installed print-function to visualize
+		** calculate cgroup-level v2 deviations
+		**
+		** allocation and deallocation of structs
+		** is arranged at a lower level
+		*/
+		if ( (supportflags&CGROUPV2) )
+			ncgroups = deviatcgroup(&devcstat, &npids);
+
+		/*
+		** activate the installed print function to visualize
 		** the deviations
 		*/
-		lastcmd = (vis.show_samp)( curtime,
+		lastcmd = (vis.show_samp)(curtime,
 				     curtime-pretime > 0 ? curtime-pretime : 1,
-		           	     &devtstat, devsstat, 
+		           	     &devtstat, devsstat,
+				     devcstat, ncgroups, npids,
 		                     nprocexit, noverflow, sampcnt==0);
 
 		/*
@@ -903,14 +973,13 @@ engine(void)
 
 		free(curtpres);
 
-		if ((supportflags & NETATOPD) && (nprocexitnet > 0)) {
-				netatop_exiterase();
-		}
+		if ((supportflags & NETATOPD) && (nprocexitnet > 0))
+			netatop_exiterase();
 
 		if (gp)
-			 free(gp);
+			free(gp);
 
-		if (lastcmd == 'r')	/* reset requested ? */
+		if (lastcmd == MRESET)	/* reset requested ? */
 		{
 			sampcnt = -1;
 
@@ -922,6 +991,9 @@ engine(void)
 			/* remove all tasks in database */
 			pdb_makeresidue();
 			pdb_cleanresidue();
+
+			/* remove current cgroup info */
+			cgwipecur();
 		}
 	} /* end of main-loop */
 }
@@ -932,20 +1004,24 @@ engine(void)
 void
 prusage(char *myname)
 {
-	printf("Usage: %s [-flags] [interval [samples]]\n",
+	printf("Usage: %s [-t [absdir]] [-flags] [interval [samples]]\n",
 					myname);
 	printf("\t\tor\n");
 	printf("Usage: %s -w  file  [-S] [-%c] [interval [samples]]\n",
-					myname, MALLPROC);
+					myname, MALLACTIVE);
 	printf("       %s -r [file] [-b [YYYYMMDD]hhmm[ss]] [-e [YYYYMMDD]hhmm[ss]] [-flags]\n",
 					myname);
 	printf("\n");
 	printf("\tgeneric flags:\n");
+	printf("\t  -t   twin mode: live measurement with possibility to review earlier samples\n");
+	printf("\t                  (raw file created in /tmp or in specific directory path)\n");
 	printf("\t  -%c  show bar graphs for system statistics\n", MBARGRAPH);
 	printf("\t  -%c  show bar graphs without categories\n", MBARMONO);
+	printf("\t  -%c  show cgroup v2 metrics\n", MCGROUPS);
+	printf("\t  -7  define cgroup v2 depth level -2 till -9 (default: -7)\n");
 	printf("\t  -%c  show version information\n", MVERSION);
-	printf("\t  -%c  show or log all processes (i.s.o. active processes "
-	                "only)\n", MALLPROC);
+	printf("\t  -%c  show all processes and cgroups (i.s.o. active only)\n",
+			MALLACTIVE);
 	printf("\t  -%c  calculate proportional set size (PSS) per process\n", 
 	                MCALCPSS);
 	printf("\t  -%c  determine WCHAN (string) per thread\n", MGETWCHAN);
@@ -1139,3 +1215,120 @@ readrc(char *path, int syslevel)
 		fclose(fp);
 	}
 }
+
+/*
+** prepare twin mode
+*/
+#define TWINNAME	"atoptwinXXXXXX"
+extern char		twindir[];
+static char		*tempname;
+
+static void
+twinprepare(void)
+{
+	char	eventbuf[1024];
+	int	tempfd;
+
+	/*
+	** consistency checks for used options
+	*/
+	if (rawreadflag)
+	{
+		fprintf(stderr, "twin mode can not be combined with -r\n");
+        	exit(42);
+	}
+
+	if (rawwriteflag)
+	{
+		fprintf(stderr, "twin mode can not be combined with -w\n");
+        	exit(42);
+	}
+
+	if (!isatty(fileno(stdout)) )	// output to pipe or file?
+	{
+		fprintf(stderr, "twin mode only for interactive use\n");
+        	exit(42);
+	}
+
+	/*
+	** created unique temporary file
+	*/
+	if (strlen(twindir) + sizeof TWINNAME + 1 >= RAWNAMESZ)
+	{
+		fprintf(stderr, "twin mode directoy path too long\n");
+        	exit(42);
+	}
+
+	tempname = malloc(strlen(twindir) + sizeof TWINNAME + 1);
+
+	ptrverify(tempname, "Malloc failed for temporary twin name\n");
+
+	sprintf(tempname, "%s/%s", twindir, TWINNAME);
+
+	if ( (tempfd = mkstemp(tempname)) == -1)
+	{
+		fprintf(stderr, "%s: ", tempname);
+		perror("twin mode file creation");
+        	exit(42);
+	}
+
+	/*
+	** create lower half as child process
+	*/
+	switch (twinpid = fork())
+	{
+	   case -1:
+		perror("fork twin process");
+        	exit(42);
+
+	   case 0:	// lower half: gather data and write to rawfile
+		rawwriteflag++;
+
+		vis.show_samp = rawwrite;
+		break;
+
+	   default:	// upper half: read from raw file and visualize
+		rawreadflag++;
+
+		/*
+		** created inotify instance to be awoken when the lower half
+		** has written a new sample to the temporary file 
+		*/
+		if ( (fdinotify = inotify_init()) == -1)
+		{
+			perror("twin mode inotify init");
+        		exit(42);
+		}
+
+		(void) inotify_add_watch(fdinotify, tempname, IN_MODIFY);
+
+		/*
+		** arrange an automic kill of the lower half
+		** at the moment that the upper half terminates
+		*/
+		atexit(twinclean);
+
+		/*
+		** wait for first sample to be written by lower half
+		*/
+		(void) read(fdinotify, eventbuf, sizeof eventbuf);
+	}
+
+	/*
+	** define current raw file name for both parent and child
+	*/
+	strncpy(rawname, tempname, RAWNAMESZ-1);
+}
+
+/*
+** kill twin process that gathers data and
+** remove the temporary raw file
+*/
+static void
+twinclean(void)
+{
+	if (twinpid)    // kill lower half process
+		kill(twinpid, SIGTERM);
+
+	(void) unlink(tempname);
+}
