@@ -44,13 +44,16 @@
 #define	TBFORMAT_INT	8
 #define	PBFORMAT	9
 #define	PBFORMAT_INT	10
-#define	OVFORMAT	11
+#define	EBFORMAT	11
+#define	EBFORMAT_INT	12
+#define	OVFORMAT	13
 
 typedef	long long	count_t;
 
 struct tstat;
 struct devtstat;
 struct sstat;
+struct cgchainer;
 struct netpertask;
 
 /* 
@@ -68,8 +71,8 @@ struct netpertask;
 
 struct visualize {
 	char	(*show_samp)  (time_t, int,
-	                struct devtstat *, struct sstat *,
-			int, unsigned int, char);
+	                struct devtstat *, struct sstat *, struct cgchainer *,
+			int, int, int, unsigned int, char);
 	void	(*show_error) (const char *, ...);
 	void	(*show_end)   (void);
 	void	(*show_usage) (void);
@@ -85,6 +88,8 @@ extern time_t   	curtime;
 extern unsigned long    interval;
 extern unsigned long	sampcnt;
 extern char      	screen;
+extern int 		fdinotify;
+extern pid_t 		twinpid;
 extern int      	linelen;
 extern char      	acctreason;
 extern char		deviatonly;
@@ -146,6 +151,7 @@ extern int		almostcrit;
 */
 char		generic_samp (time_t, int,
 		            struct devtstat *, struct sstat *,
+			    struct cgchainer *, int, int,
 		            int, unsigned int, char);
 void		generic_error(const char *, ...);
 void		generic_end  (void);
@@ -190,6 +196,7 @@ int		llccompar(const void *, const void *);
 int  		rawread(void);
 char		rawwrite (time_t, int,
 		            struct devtstat *, struct sstat *,
+			    struct cgchainer *, int, int,
 		            int, unsigned int, char);
 
 int 		numeric(char *);
