@@ -55,10 +55,6 @@
 			"%*d  %*d  %*d  %*d  %*d %*d  "	\
 			"%d   %d   %d   %lld"
 
-/* ATOP-extension line of /proc/pid/stat */
-#define ATOPSTAT	"%lld %llu %lld %llu %lld %llu %lld %llu "	\
-			"%lld %llu %lld %llu %lld %lld"
-
 static int	procstat(struct tstat *, unsigned long long, char);
 static int	procstatus(struct tstat *);
 static int	procio(struct tstat *);
@@ -66,14 +62,6 @@ static void	proccmd(struct tstat *);
 static void	procsmaps(struct tstat *);
 static void	procwchan(struct tstat *);
 static count_t	procschedstat(struct tstat *);
-static int	proccgroupv2(struct tstat *);
-static struct cgroupv2vals *
-		alloccgroupv2(char *, int);
-static struct cgroupv2vals *
-		findhashcgroupv2(char *, int *);
-void		fillcgroupv2(struct cgroupv2vals *, char *, char *, int);
-int 		readcgroupv2(char *, char *, char *, int, long []);
-static void	wipecgroupv2(void);
 
 extern GHashTable *ghash_net;
 
@@ -92,7 +80,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 	FILE		*fp;
 	DIR		*dirp;
 	struct dirent	*entp;
-	char		origdir[1024], dockstat=0;
+	char		origdir[4096], dockstat=0;
 	unsigned long	tval=0;
 
 	/*
@@ -111,25 +99,6 @@ photoproc(struct tstat *tasklist, int maxtask)
 			fclose(fp);
 		}
 
-		/*
-		** check if this kernel offers cgroups version 2
-		*/
-		if ( (fp = fopen("/proc/1/cgroup", "r")) )
-		{
-			char line[128];
-
-			while (fgets(line, sizeof line, fp))
-			{
-				if (memcmp(line, "0::", 3) == 0) // equal?
-				{
-					supportflags |= CGROUPV2;
-					break;
-				}
-			}
-
-			fclose(fp);
-		}
-
 		if (! droprootprivs())
 			mcleanstop(42, "failed to drop root privs\n");
 
@@ -169,9 +138,6 @@ photoproc(struct tstat *tasklist, int maxtask)
 		netatop_bpf_gettask();
 	}
 
-	if (supportflags & CGROUPV2)
-		wipecgroupv2();
-
 	if (! droprootprivs())
 		mcleanstop(42, "failed to drop root privs\n");
 
@@ -230,13 +196,6 @@ photoproc(struct tstat *tasklist, int maxtask)
 		proccmd(curtask);			/* from /proc/pid/cmdline */
 		dockstat += getutsname(curtask);	/* retrieve container/pod name */
 
-		/*
- 		** cgroups v2: determine cgroup for process and register most
-		**             relevant limits
-		*/
-		if (supportflags & CGROUPV2)
-			proccgroupv2(curtask);
-
 		/*
 		** reading the smaps file for every process with every sample
 		** is a really 'expensive' from a CPU consumption point-of-view,
@@ -715,12 +674,14 @@ procstatus(struct tstat *curtask)
 			continue;
 		}
 
-		if (memcmp(line, "voluntary_ctxt_switches:", 24)==0) {
+		if (memcmp(line, "voluntary_ctxt_switches:", 24)==0)
+		{
 			sscanf(line, "voluntary_ctxt_switches: %lld", &(curtask->cpu.nvcsw));
 			continue;
 		}
 
-		if (memcmp(line, "nonvoluntary_ctxt_switches:", 27)==0) {
+		if (memcmp(line, "nonvoluntary_ctxt_switches:", 27)==0)
+		{
 			sscanf(line, "nonvoluntary_ctxt_switches: %lld", &(curtask->cpu.nivcsw));
 			continue;
 		}
@@ -801,7 +762,7 @@ procio(struct tstat *curtask)
 **    - tabs (e.g. arguments for awk or sed)
 ** these special bytes will be converted to spaces
 **
-** the command line may be prepended by environmentvariables
+** the command line may be prepended by environment variables
 */
 
 #define	ABBENVLEN	16
@@ -1025,405 +986,3 @@ procschedstat(struct tstat *curtask)
 
 	return curtask->cpu.rundelay;
 }
-
-/*
-** CGROUP V2 specific items
-*/
-#define	CGROUPROOT	"/sys/fs/cgroup"
-#define	CGROUPNHASH	64
-#define	CGROUPMASK	0x3f
-#define	MAXSLASH	16	// max. number of slashes in relative path
-
-struct cgroupv2vals {
-	char			*path;
-
-	int			cpuweight;  	// -1=max, -2=undefined
-
-	int			cpumax;  	// -1=max, -2=undefined (perc)
-	int			cpumaxr;  	// -1=max, -2=undefined (perc)
-
-	long long		memmax;		// -1=max, -2=undefined (KiB)
-	long long		memmaxr;	// -1=max, -2=undefined (KiB)
-
-	long long		swpmax;		// -1=max, -2=undefined (KiB)
-	long long		swpmaxr;	// -1=max, -2=undefined (KiB)
-
-	struct cgroupv2vals	*next;
-};
-
-static struct cgroupv2vals *cgrouphash[CGROUPNHASH];
-
-/*
-** get cgroup related to process from /proc/<pid>/cgroup
-** return code:		0 - no cgroup v2 in use
-**			1 - cgroup v2 in use
-**			2 - cgroup version can not be determined
-*/
-static int
-proccgroupv2(struct tstat *curtask)
-{
-	FILE			*fp;
-	char			line[1024], *relpath, abspath[1200];
-	int			hash, pathlen, restlen, nslash;
-	struct cgroupv2vals 	*pvals = NULL, *ptarget;
-	char			*p, *slashes[MAXSLASH];
-
-	/*
- 	** open the cgroup file of the current process and
-	** read the line that starts with '0::' for cgroup v2
-	*/
-	curtask->gen.cgpath[0] = '\0';
-
-	if ( (fp = fopen("cgroup", "r")) )
-	{
-		while (fgets(line, sizeof line, fp))
-		{
-			if ( memcmp(line, "0::", 3) == 0) // equal?
-			{
-				line[ strlen(line)-1 ] = '\0';	// remove newline
-
-				relpath = line+3;
-
-				strncpy(curtask->gen.cgpath, relpath,
-						sizeof curtask->gen.cgpath);
-
-				curtask->gen.cgpath[sizeof curtask->gen.cgpath -1] = '\0';
-
-				break;
-			}
-		}
-
-		fclose(fp);
-
-		if (curtask->gen.cgpath[0] == '\0')
-			return 0;		// no cgroupv2 support
-	}
-	else	// open failed; no permission
-	{
-		return 2;
-	}
-
-	/*
-	** cgroup v2 pathname of this process is known;
-	** prepare absolute pathname of cgroup
-	*/
-	pathlen = snprintf(abspath, sizeof abspath, "%s%s/",
-						CGROUPROOT, relpath);
-	restlen = sizeof abspath - pathlen -1;
-	abspath[sizeof abspath - 1] = '\0';	// guarantee delimiter
-	relpath = abspath + sizeof CGROUPROOT - 1;
-
-	/*
-	** cycle through all directory levels for values that
-	** might limit the values in the current cgroup (e.g. cpu.max)
-	*/
-	for (nslash=0, p=relpath; *p && nslash<MAXSLASH; p++)
-	{
-		if (*p == '/')			// find all slashes in path
-			slashes[nslash++] = p;
-	}
-
-	for (nslash--, ptarget=NULL; nslash > 0; nslash--)
-	{
-		*slashes[nslash] = '\0';
-
-		pvals = findhashcgroupv2(relpath, &hash); // search in cache
-
-		if (!pvals)				// not found in cache
-		{
-			// allocate new cache entry
-        		pvals = alloccgroupv2(relpath, hash);
-
-			// fill info in new cache entry
-			fillcgroupv2(pvals, abspath, slashes[nslash],
-								restlen);
-		}
-
-
-		/*
-		** if the target cgroup is not defined,
-		** determine the target cgroup (lowest level dir)
-		*/
-		if (! ptarget)
-		{
-			ptarget = pvals;
-			continue;
-		}
-
-		/*
-		** in case of a higher cgroup, check the restrictive values
-		** for the target cgroup
-		*/
-		switch (pvals->cpumax)
-		{
-		   case -1:
-			if (ptarget->cpumaxr == -2)
-				ptarget->cpumaxr = -1;
-			break;
-		   case -2:
-			break;
-		   default:
-			if (ptarget->cpumaxr == -1 || ptarget->cpumaxr == -2)
-			{
-				ptarget->cpumaxr = pvals->cpumax;
-				break;
-			}
-
-			if (ptarget->cpumaxr > pvals->cpumax)
-				ptarget->cpumaxr = pvals->cpumax;
-		}
-
-		switch (pvals->memmax)
-		{
-		   case -1:
-			if (ptarget->memmaxr == -2)
-				ptarget->memmaxr = -1;
-			break;
-		   case -2:
-			break;
-		   default:
-			if (ptarget->memmaxr == -1 || ptarget->memmaxr == -2)
-			{
-				ptarget->memmaxr = pvals->memmax;
-				break;
-			}
-
-			if (ptarget->memmaxr > pvals->memmax)
-				ptarget->memmaxr = pvals->memmax;
-		}
-
-		switch (pvals->swpmax)
-		{
-		   case -1:
-			if (ptarget->swpmaxr == -2)
-				ptarget->swpmaxr = -1;
-			break;
-		   case -2:
-			break;
-		   default:
-			if (ptarget->swpmaxr == -1 || ptarget->swpmaxr == -2)
-			{
-				ptarget->swpmaxr = pvals->swpmax;
-				break;
-			}
-
-			if (ptarget->swpmaxr > pvals->swpmax)
-				ptarget->swpmaxr = pvals->swpmax;
-		}
-
-
-	}
-
-	curtask->cpu.cgcpuweight = ptarget->cpuweight;
-
-	curtask->cpu.cgcpumax    = ptarget->cpumax;
-	curtask->cpu.cgcpumaxr   = ptarget->cpumaxr;
-
-	curtask->mem.cgmemmax    = ptarget->memmax;
-	curtask->mem.cgmemmaxr   = ptarget->memmaxr;
-
-	curtask->mem.cgswpmax    = ptarget->swpmax;
-	curtask->mem.cgswpmaxr   = ptarget->swpmaxr;
-
-	return 1;
-}
-
-/*
-** determine the most relevant values of this cgroup
-*/
-void
-fillcgroupv2(struct cgroupv2vals *pvals, char *abspath, char *extpath,
-								int restlen)
-{
-	long	retvals[2];
-
-	*extpath++ = '/';			// replace slash
-
-	/*
-	** get cpu.weight limitation
-	*/
-	pvals->cpuweight = -2;			// initial value (undefined)
-
-	switch (readcgroupv2(abspath, extpath, "cpu.weight", restlen, retvals))
-	{
-	   case 1:
-		pvals->cpuweight = retvals[0];
-		break;
-	}
-
-	/*
-	** get cpu.max limitation
-	*/
-	pvals->cpumax = -2;			// initial value (undefined)
-
-	switch (readcgroupv2(abspath, extpath, "cpu.max", restlen, retvals))
-	{
-	   case 2:
-		if (retvals[0] == -1)
-			pvals->cpumax = -1;
-		else
-			pvals->cpumax = retvals[0] * 100 / retvals[1];
-		break;
-	}
-
-	pvals->cpumaxr = pvals->cpumax;		// set temporary restrictive
-
-	/*
-	** get memory.max limitation
-	*/
-	pvals->memmax = -2;			// initial value (undefined)
-
-	switch (readcgroupv2(abspath, extpath, "memory.max", restlen, retvals))
-	{
-	   case 1:
-		if (retvals[0] == -1)
-			pvals->memmax = -1;
-		else
-			pvals->memmax = retvals[0] / 1024;	// KiB
-		break;
-	}
-
-	pvals->memmaxr = pvals->memmax;		// set temporary restrictive
-
-	/*
-	** get memory.swap.max limitation
-	*/
-	pvals->swpmax = -2;			// initial value (undefined)
-
-	switch (readcgroupv2(abspath, extpath, "memory.swap.max", restlen, retvals))
-	{
-	   case 1:
-		if (retvals[0] == -1)
-			pvals->swpmax = -1;
-		else
-			pvals->swpmax = retvals[0] / 1024;	// KiB
-		break;
-	}
-
-	pvals->swpmaxr = pvals->swpmax;		// set temporary restrictive
-}
-
-/*
-** read line with one or two values
-** and fill one or (maximum) two values
-** in retvals (value 'max' converted into -1)
-**
-** return value:	number of entries in retvals filled
-*/
-int
-readcgroupv2(char *abspath, char *extpath, char *fname, int restlen,
-							long retvals[])
-{
-	char line[64];
-	int  n;
-	FILE *fp;
-
-	strncpy(extpath, fname, restlen);  // complete absolute path of file
-
-	if ( (fp = fopen(abspath, "r")) )
-	{
-		char firststr[16];
-
-		if (! fgets(line, sizeof line, fp))
-		{
-			fclose(fp);
-			return 0;
-		}
-
-		fclose(fp);
-
-		switch (n = sscanf(line, "%15s %ld", firststr, &retvals[1]))
-		{
-		   case 0:
-			return 0;
-		   case 1:
-		   case 2:
-			if ( strcmp(firststr, "max") == 0)
-				retvals[0] = -1;
-			else
-				retvals[0] = atol(firststr);
-
-			return n;
-		   default:
-			return 0;
-		}
-	}
-
-	return 0;
-}
-
-/*
-** find existing info about cgroup in cgroupv2 cache
-**
-** return value:	pointer to structure that has been found, or
-**			NULL (not found)
-*/
-static struct cgroupv2vals *
-findhashcgroupv2(char *relpath, int *phash)
-{
-	struct cgroupv2vals 	*p;
-	char 			*s;
-	int  			hash = 0;
-
-	for (s = relpath; *s++; )	// calculate simple hash for this cgroup
-		hash += *s;		// by accumulating all path characters
-
-	*phash = hash;
-
-	// search hash list of earlier accessed cgroups within this interval
-	for (p = cgrouphash[hash&CGROUPMASK]; p; p = p->next)
-	{
-		if ( strcmp(relpath, p->path) == 0)	// found?
-			return p;
-	}
-
-	return NULL;
-}
-
-/*
-** allocate new hash entry in cgroupv2 cache
-**
-** return value:	pointer to newly allocated structure
-*/
-static struct cgroupv2vals *
-alloccgroupv2(char *relpath, int hash)
-{
-	struct cgroupv2vals 	*p;
-
-	p = malloc(sizeof(struct cgroupv2vals));
-
-	ptrverify(p, "Malloc failed for new cgroup values\n");
-
-	p->path = malloc(strlen(relpath)+1);
-
-	ptrverify(p->path, "Malloc failed for path in new cgroup values\n");
-
-	strcpy(p->path, relpath);
-
-	p->next = cgrouphash[hash&CGROUPMASK]; 	// add new entry to hash chain
-	cgrouphash[hash&CGROUPMASK] = p;
-
-	return p;
-}
-
-/*
-** clear entire cgroupv2 cache
-*/
-static void
-wipecgroupv2(void)
-{
-	int i;
-	struct cgroupv2vals *p, *pnext;
-
-	for (i=0; i < CGROUPNHASH; i++)
-	{
-		for (p = cgrouphash[i]; p; p = pnext)
-		{
-			pnext = p->next;
-			free(p->path);
-			free(p);
-		}
-
-		cgrouphash[i] = 0;
-	}
-}
